@@ -33,12 +33,14 @@ else:
 
 # a flag we can check elsewhere for Python 3
 PY3 = sys.version_info.major >= 3
+
 if PY3:
     # for type checking
     basestring = str
     # Python 3
     from io import BytesIO, StringIO
     from shutil import which  # noqa
+    from time import perf_counter as now  # noqa
 else:
     # Python 2
     from StringIO import StringIO
@@ -47,6 +49,7 @@ else:
     StringIO.__enter__ = lambda a: a
     StringIO.__exit__ = lambda a, b, c, d: a.close()
     BytesIO = StringIO
+    from time import time as now  # noqa
 
 
 try:
@@ -67,6 +70,9 @@ TOL_MERGE = 1e-8
 # enable additional potentially slow checks
 _STRICT = False
 
+_IDENTITY = np.eye(4, dtype=np.float64)
+_IDENTITY.flags['WRITEABLE'] = False
+
 
 def has_module(name):
     """
@@ -568,7 +574,7 @@ try:
     # only included in recent-ish version of numpy
     multi_dot = np.linalg.multi_dot
 except AttributeError:
-    log.warning('np.linalg.multi_dot not available, using fallback')
+    log.debug('np.linalg.multi_dot not available, using fallback')
 
     def multi_dot(arrays):
         """
@@ -1165,7 +1171,8 @@ def structured_array_to_string(array,
 
     # abort for unstructured arrays
     if array.dtype.names is None:
-        raise ValueError('array is not structured, use array_to_string instead')
+        raise ValueError(
+            'array is not structured, use array_to_string instead')
 
     # do not allow a value to be repeated in a value format
     if value_format.count('{') > 1:
@@ -1404,9 +1411,7 @@ def type_bases(obj, depth=4):
         bases = np.hstack(bases)
     except IndexError:
         bases = []
-    # we do the hasattr as None/NoneType can be in the list of bases
-    bases = [i for i in bases if hasattr(i, '__name__')]
-    return np.array(bases)
+    return [i for i in bases if hasattr(i, '__name__')]
 
 
 def type_named(obj, name):
@@ -1416,12 +1421,15 @@ def type_named(obj, name):
 
     Parameters
     ------------
-    obj: object to look for class of
-    name : str, name of class
+    obj : any
+      Object to look for class of
+    name : str
+      Nnme of class
 
     Returns
     ----------
-    named class, or None
+    class : Optional[Callable]
+      Camed class, or None
     """
     # if obj is a member of the named class, return True
     name = str(name)
@@ -1450,47 +1458,66 @@ def concatenate(a, b=None):
     result : trimesh.Trimesh
       Concatenated mesh
     """
-    if b is None:
-        b = []
-    # stack meshes into flat list
-    meshes = np.append(a, b)
 
-    # if there is only one mesh just return the first
-    if len(meshes) == 1:
-        return meshes[0].copy()
-    elif len(meshes) == 0:
+    # get a flat list of meshes
+    flat = []
+    if a is not None:
+        if is_sequence(a):
+            flat.extend(a)
+        else:
+            flat.append(a)
+    if b is not None:
+        if is_sequence(b):
+            flat.extend(b)
+        else:
+            flat.append(b)
+
+    if len(flat) == 1:
+        # if there is only one mesh just return the first
+        return flat[0].copy()
+    elif len(flat) == 0:
+        # if there are no meshes return an empty list
+        return []
+
+    is_mesh = [f for f in flat if is_instance_named(f, 'Trimesh')]
+    is_path = [f for f in flat if is_instance_named(f, 'Path')]
+
+    if len(is_path) > len(is_mesh):
+        from .path.util import concatenate as concatenate_path
+        return concatenate_path(is_path)
+
+    if len(is_mesh) == 0:
         return []
 
     # extract the trimesh type to avoid a circular import
-    # and assert that both inputs are Trimesh objects
-    trimesh_type = type_named(meshes[0], 'Trimesh')
+    # and assert that all inputs are Trimesh objects
+    trimesh_type = type_named(is_mesh[0], 'Trimesh')
 
     # append faces and vertices of meshes
     vertices, faces = append_faces(
-        [m.vertices.copy() for m in meshes],
-        [m.faces.copy() for m in meshes])
+        [m.vertices.copy() for m in is_mesh],
+        [m.faces.copy() for m in is_mesh])
 
     # only save face normals if already calculated
     face_normals = None
-    if all('face_normals' in m._cache for m in meshes):
+    if all('face_normals' in m._cache for m in is_mesh):
         face_normals = np.vstack(
-            [m.face_normals for m in meshes])
+            [m.face_normals for m in is_mesh])
 
     try:
         # concatenate visuals
-        visual = meshes[0].visual.concatenate(
-            [m.visual for m in meshes[1:]])
+        visual = is_mesh[0].visual.concatenate(
+            [m.visual for m in is_mesh[1:]])
     except BaseException:
-        log.debug(
-            'failed to combine visuals', exc_info=True)
+        log.debug('failed to combine visuals', exc_info=True)
         visual = None
+
     # create the mesh object
-    mesh = trimesh_type(vertices=vertices,
+    return trimesh_type(vertices=vertices,
                         faces=faces,
                         face_normals=face_normals,
                         visual=visual,
                         process=False)
-    return mesh
 
 
 def submesh(mesh,
@@ -1563,7 +1590,11 @@ def submesh(mesh,
         normals.append(mesh.face_normals[index])
         faces.append(mask[current])
         vertices.append(original_vertices[unique])
-        visuals.append(mesh.visual.face_subset(index))
+
+        try:
+            visuals.append(mesh.visual.face_subset(index))
+        except BaseException:
+            visuals = None
 
     if len(vertices) == 0:
         return np.array([])
@@ -1588,6 +1619,9 @@ def submesh(mesh,
             process=False)
         return appended
 
+    if visuals is None:
+        visuals = [None] * len(vertices)
+
     # generate a list of Trimesh objects
     result = [trimesh_type(
         vertices=v,
@@ -1887,27 +1921,19 @@ def decompress(file_obj, file_type):
       Data from archive in format {file name : file-like}
     """
 
-    def is_zip():
-        archive = zipfile.ZipFile(file_obj)
-        result = {name: wrap_as_stream(archive.read(name))
-                  for name in archive.namelist()}
-        return result
-
-    def is_tar():
-        import tarfile
-        archive = tarfile.open(fileobj=file_obj, mode='r')
-        result = {name: archive.extractfile(name)
-                  for name in archive.getnames()}
-        return result
-
     file_type = str(file_type).lower()
     if isinstance(file_obj, bytes):
         file_obj = wrap_as_stream(file_obj)
 
-    if file_type[-3:] == 'zip':
-        return is_zip()
+    if file_type.endswith('zip'):
+        archive = zipfile.ZipFile(file_obj)
+        return {name: wrap_as_stream(archive.read(name))
+                for name in archive.namelist()}
     if 'tar' in file_type[-6:]:
-        return is_tar()
+        import tarfile
+        archive = tarfile.open(fileobj=file_obj, mode='r')
+        return {name: archive.extractfile(name)
+                for name in archive.getnames()}
     raise ValueError('Unsupported type passed!')
 
 
@@ -1946,7 +1972,7 @@ def compress(info, **kwargs):
     return compressed
 
 
-def split_extension(file_name, special=['tar.bz2', 'tar.gz']):
+def split_extension(file_name, special=None):
     """
     Find the file extension of a file name, including support for
     special case multipart file extensions (like .tar.gz)
@@ -1967,6 +1993,8 @@ def split_extension(file_name, special=['tar.bz2', 'tar.gz']):
     """
     file_name = str(file_name)
 
+    if special is None:
+        special = ['tar.bz2', 'tar.gz']
     if file_name.endswith(tuple(special)):
         for end in special:
             if file_name.endswith(end):
@@ -2310,7 +2338,6 @@ def decode_text(text, initial='utf-8'):
     # if not bytes just return input
     if not hasattr(text, 'decode'):
         return text
-
     try:
         # initially guess file is UTF-8 or specified encoding
         text = text.decode(initial)
@@ -2318,15 +2345,17 @@ def decode_text(text, initial='utf-8'):
         # detect different file encodings
         import chardet
         # try to detect the encoding of the file
-        detect = chardet.detect(text)
+        # only look at the first 1000 characters otherwise
+        # for big files chardet looks at everything and is slow
+        detect = chardet.detect(text[:1000])
         # warn on files that aren't UTF-8
-        log.warning(
+        log.debug(
             'Data not {}! Trying {} (confidence {})'.format(
                 initial,
                 detect['encoding'],
                 detect['confidence']))
         # try to decode again, unwrap in try
-        text = text.decode(detect['encoding'])
+        text = text.decode(detect['encoding'], errors='ignore')
     return text
 
 
@@ -2355,7 +2384,7 @@ def to_ascii(text):
     return str(text)
 
 
-def is_ccw(points):
+def is_ccw(points, return_all=False):
     """
     Check if connected 2D points are counterclockwise.
 
@@ -2363,22 +2392,102 @@ def is_ccw(points):
     -----------
     points : (n, 2) float
       Connected points on a plane
+    return_all : bool
+      Return polygon area and centroid or just counter-clockwise.
 
     Returns
     ----------
     ccw : bool
       True if points are counter-clockwise
+    area : float
+      Only returned if `return_centroid`
+    centroid : (2,) float
+      Centroid of the polygon.
     """
-    points = np.asanyarray(points, dtype=np.float64)
+    points = np.array(points, dtype=np.float64)
 
-    if (len(points.shape) != 2 or points.shape[1] != 2):
-        raise ValueError('CCW is only defined for 2D')
-    xd = np.diff(points[:, 0])
-    # sum along axis=1 with a dot product
-    yd = np.dot(np.column_stack((
-        points[:, 1],
-        points[:, 1])).reshape(-1)[1:-1].reshape((-1, 2)), [1, 1])
-    area = np.sum(xd * yd) * .5
-    ccw = area < 0
+    if len(points.shape) != 2 or points.shape[1] != 2:
+        raise ValueError('only defined for `(n, 2)` points')
 
-    return ccw
+    # the "shoelace formula"
+    product = np.subtract(*(points[:-1, [1, 0]] * points[1:]).T)
+    # the area of the polygon
+    area = product.sum() / 2.0
+    # check the sign of the area
+    ccw = area < 0.0
+
+    if not return_all:
+        return ccw
+
+    # the centroid of the polygon uses the same formula
+    centroid = ((points[:-1] + points[1:]) *
+                product.reshape((-1, 1))).sum(axis=0) / (6.0 * area)
+
+    return ccw, area, centroid
+
+
+def unique_name(start, contains, counts=None):
+    """
+    Deterministically generate a unique name not
+    contained in a dict, set or other grouping with
+    `__includes__` defined. Will create names of the
+    form "start_10" and increment accordingly.
+
+    Parameters
+    -----------
+    start : str
+      Initial guess for name.
+    contains : dict, set, or list
+      Bundle of existing names we can *not* use.
+    counts : None or dict
+      Maps name starts encountered before to increments in
+      order to speed up finding a unique name as otherwise
+      it potentially has to iterate through all of contains.
+      Should map to "how many times has this `start`
+      been attempted, i.e. `counts[start]: int`.
+      Note that this *will be mutated* in-place by this function!
+
+    Returns
+    ---------
+    unique : str
+      A name that is not contained in `contains`
+    """
+    # exit early if name is not in bundle
+    if (start is not None and
+        len(start) > 0 and
+            start not in contains):
+        return start
+
+    # start checking with zero index unless found
+    if counts is None:
+        increment = 0
+    else:
+        increment = counts.get(start, 0)
+    if start is not None and len(start) > 0:
+        formatter = start + '_{}'
+        # split by our delimiter once
+        split = start.rsplit('_', 1)
+        if len(split) == 2 and increment == 0:
+            try:
+                # start incrementing from the existing
+                # trailing value
+                # if it is not an integer this will fail
+                increment = int(split[1])
+                # include the first split value
+                formatter = split[0] + '_{}'
+            except BaseException:
+                pass
+    else:
+        formatter = 'geometry_{}'
+
+    # if contains is empty we will only need to check once
+    for i in range(increment + 1, 2 + increment + len(contains)):
+        check = formatter.format(i)
+        if check not in contains:
+            if counts is not None:
+                counts[start] = i
+            return check
+
+    # this should really never happen since we looped
+    # through the full length of contains
+    raise ValueError('Unable to establish unique name!')
